def var ch1 as char label "Numeric format".
def var ch2 as char label "Decimal point".
def var ch3 as char label "Numeric separator".
def var caz as int label "Case #".
def var d as dec label "Decimal" init 12345678.09 format ">>>,>>>,>>>.99".

def var sch1 as char.
def var sch2 as char.

form caz ch1 ch2 ch3 d with frame f1 down title "Numeric separators".

function checkMe returns int (i as int, ns as char, ds as char, nf as char, df as char).
   display i @ caz nf @ ch1 ds @ ch2 ns @ ch3 d with frame f1.
  
   if (session:numeric-decimal-point <> ds)
     then message string(i) ": expected decimal point " ds " and found " session:numeric-decimal-point.
     
   if (session:numeric-separator <> ns)
     then message string(i) ": expected numeric separator " ns " and found " session:numeric-separator.
  
   if (session:numeric-format <> nf)
     then message string(i) ": expected numeric format " nf " and found " session:numeric-format.
     
   def var sdf as char.
   sdf = trim(string(d, ">>>,>>>,>>>.99")).
   if (sdf <> df)
     then message string(i) ": expected " df " and found " sdf.
     
   sdf = d:screen-value in frame f1.
   if (sdf <> df)
     then message string(i) ": expected " df " and found screen-value " sdf.

   down with frame f1.
end.

function checkError returns int(caz as int, err as log, num as int, msg as char, code as int).
   if (error-status:error <> err)
    then message string(caz) " error status is " string(error-status:error) " and expected is " string(err) "".

   if (error-status:num-messages <> num)
    then message string(caz) " num messages is " string(error-status:num-messages) " and expected is " string(num) "".

   if (error-status:get-message(1) <> msg)
    then message string(caz) " message 1 is " error-status:get-message(1) " and expected is " msg "".

   if (error-status:get-number(1) <> code)
    then message string(caz) " message code 1 is " string(error-status:get-number(1)) " and expected is " string(code) "".
end.

/* initial values */
checkMe(0, session:numeric-separator, session:numeric-decimal-point, session:numeric-format, "12,345,678.09").

sch1 = ?.
sch2 = ?.
session:set-numeric-format(sch1, sch2) no-error.
checkError(1, no, 1, "An invalid character '' (0) was specified as the numeric separator or decimal point. (8903)", 8903).
checkMe(1, ",", ".", "AMERICAN", "12,345,678.09").

sch1 = ".".
sch2 = ?.
session:set-numeric-format(sch1, sch2) no-error.
checkError(2, no, 1, "An invalid character '' (0) was specified as the numeric separator or decimal point. (8903)", 8903).
checkMe(2, ",", ".", "AMERICAN", "12,345,678.09").

sch1 = ?.
sch2 = ",".
session:set-numeric-format(sch1, sch2) no-error.
checkError(2, no, 1, "An invalid character '' (0) was specified as the numeric separator or decimal point. (8903)", 8903).
checkMe(2, ",", ".", "AMERICAN", "12,345,678.09").

/* invalid characters */
def var invalidSepChars as char init "B,C,D,R,Z,z,0,1,2,3,4,5,6,7,8,9,+,-,<,>,(,),*,?,DD".
def var invalidDepChars as char init "B,C,D,R,Z,z,0,1,2,3,4,5,6,7,8,9,+,-,<,>,(,),*,?, ,DD".
def var i as int.
def var msg as char.

do i = 1 to num-entries(invalidSepChars, ","):
   sch1 = entry(i, invalidSepChars, ",").
   sch2 = session:numeric-decimal-point.
   session:set-numeric-format(sch1, sch2) no-error.
   msg = "An invalid character '" + sch1 + "' (" + string(asc(substring(sch1, 1, 1))) + ") was specified as the numeric separator or decimal point. (8903)".
   checkError(2 + i, no, 1, msg, 8903).
   checkMe(2 + i, ",", ".", "AMERICAN", "12,345,678.09").
end.

do i = 1 to num-entries(invalidDepChars, ","):
   sch1 = session:numeric-separator.
   sch2 = entry(i, invalidDepChars, ",").
   session:set-numeric-format(sch1, sch2) no-error.
   msg = "An invalid character '" + sch2 + "' (" + string(asc(substring(sch2, 1, 1))) + ") was specified as the numeric separator or decimal point. (8903)".
   checkError(27 + i, no, 1, msg, 8903).
   checkMe(27 + i, ",", ".", "AMERICAN", "12,345,678.09").
end.

sch1 = "$".
sch2 = "$".
session:set-numeric-format(sch1, sch2) no-error.
checkError(54, no, 1, "The numeric separator and decimal point may not be the same character. (8904)", 8904).
checkMe(54, ",", ".", "AMERICAN", "12,345,678.09").

sch1 = ",".
sch2 = ",".
session:set-numeric-format(sch1, sch2) no-error.
checkError(55, no, 1, "The numeric separator and decimal point may not be the same character. (8904)", 8904).
checkMe(55, ",", ".", "AMERICAN", "12,345,678.09").

sch1 = "$".
sch2 = "#".
session:set-numeric-format(sch1, sch2) no-error.
checkError(56, no, 0, "", 0).
checkMe(56, "$", "#", "$#", "12$345$678#09").

sch1 = ",".
sch2 = ".".
session:set-numeric-format(sch1, sch2) no-error.
checkError(57, no, 0, "", 0).
checkMe(57, ",", ".", "AMERICAN", "12,345,678.09").

sch1 = ".".
sch2 = ",".
session:set-numeric-format(sch1, sch2) no-error.
checkError(58, no, 0, "", 0).
checkMe(58, ".", ",", "EUROPEAN", "12.345.678,09").

sch1 = "".
sch2 = "".
session:set-numeric-format(sch1, sch2) no-error.
checkError(59, no, 1, "An invalid character '' (0) was specified as the numeric separator or decimal point. (8903)", 8903).
checkMe(59, ".", ",", "EUROPEAN", "12.345.678,09").
